///////////////////////////////////////////////////////////////////////
// ADSAutorun.cpp
//
// Applied Data Systems
// ------------------------------------------------------------------
// 720020-1145x
//
// Description
// -----------
// ADSAutorun is a utility to allow a user to automatically run an application when
// an ATA card is inserted into an ADS device.  The following are the rules for 
// ADSAutorun's operation:
//
// 1. ADSAutorun must be run once to register for card notifications.  If you use
//    it as ADSLOAD.EXE, it will automatically run on boot.  This is the recommended
//    usage.
// 2. When an ATA card is inserted, ADSAutorun searches the card for the following
//    file:  "\Storage Card{n}\autorun.exe".  This allows autorun to work in either
//    slot on boards with multiples.
// 3. If the file exists, it is copies to the local filesystem at the following
//    location: "\temp\autorun{n}.exe" where n is the slot it was copied from.
// 4. The file is run with a command line of "/INSERT" this allows the application
//    to react appropriately.
// 5. On card removal the local temp version on autorun.exe is run, this time with 
//    a command line of "/REMOVE".  This allows the application to do any necessary
//    cleanup.  If no action is desired, you still *must* handle this case because 
//    the application is still called.  ADSAutorun *blocks* at this point!
// 6. Once the app completes, ADSAutorun deletes the temporary file.
//
//
// History
// -------
// 720020-11451 - April 1, 2003 : ctacke
//   - Initial implementation
//

#include <windows.h>
#include "notify.h"

#define APP_NAME		_T("ADSAutorun")
#define TEMP_FILE_NAME1	_T("\\temp\\autorun.exe")
#define TEMP_FILE_NAME2	_T("\\temp\\autorun2.exe")

short WhichCard(TCHAR *lpCmdLine);

int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	HKEY	hKey	= NULL;
	DWORD	dwDisp	= 0;
	DWORD	dwData	= 0;
	DWORD	dwType	= REG_DWORD;
	DWORD	dwSize	= sizeof(DWORD);
	short	sCard	= 0;
	TCHAR	szAppPath[MAX_PATH];
	PROCESS_INFORMATION pi;

	// check the registry to see if we've registered for the notification
	RegCreateKeyEx(HKEY_LOCAL_MACHINE, _T("ADS"), 0, NULL, 0, 0, NULL, &hKey, &dwDisp);
	if(ERROR_SUCCESS != RegQueryValueEx(hKey, APP_NAME, 0, &dwType, (BYTE*)&dwData, &dwSize))
	{
		// first run, so register event
		GetModuleFileName(NULL, szAppPath, MAX_PATH);
		DEBUGMSG(TRUE, (_T("ADSAutorun registering %s\r\n"), szAppPath));
		CeRunAppAtEvent(szAppPath, NOTIFICATION_EVENT_DEVICE_CHANGE);

		// set the registry key
		dwData = 1;
		RegSetValueEx(hKey, APP_NAME, 0, REG_DWORD, (BYTE*)&dwData, sizeof(dwData));
		RegCloseKey(hKey);

		return 0;
	}

	TCHAR *p;
	// was the card inserted or deleted?
	p = _tcsstr(lpCmdLine, _T("/"));
	if(p == NULL)
	{
		// no command line, so it was run by the user
		DEBUGMSG(TRUE, (_T("ADSAutorun already registered\r\n")));
		return 0;
	}

	sCard = WhichCard(lpCmdLine);
	if(sCard == 0)
	{
		// not an ATA card
		DEBUGMSG(TRUE, (_T("ADSAutorun: not an ATA card\r\n")));
		return 0;
	}

	switch(p[1])
	{
		case 'A':
			DEBUGMSG(TRUE, (_T("ADSAutorun Card%i Insert Detected\r\n"), sCard));

			// get path to card file
			_tcscpy(szAppPath, _T("\\Storage Card"));
			if(sCard == 2)
			{
				_tcscat(szAppPath, _T("2"));
			}

			// let the card filesystem get mounted
			while(GetFileAttributes(szAppPath) == MAXDWORD)
			{
				Sleep(100);
			}
			
			_tcscat(szAppPath, _T("\\ADS\\Autorun.exe"));

			// copy the autorun app locally
			if(!CopyFile(szAppPath, (sCard == 1) ? TEMP_FILE_NAME1 : TEMP_FILE_NAME2, FALSE))
			{
				if(GetLastError() == ERROR_PATH_NOT_FOUND || GetLastError() == ERROR_FILE_NOT_FOUND)
				{
					// nothing to do
					DEBUGMSG(TRUE, (_T("ADSAutorun: no app to run\r\n")));
					return 0;
				}
				
				RETAILMSG(TRUE, (_T("ADSAutorun failed with error code %i\r\n"), GetLastError()));
				return -1;
			}

			// run the app
			if(!CreateProcess((sCard == 1) ? TEMP_FILE_NAME1 : TEMP_FILE_NAME2, _T("/INSERT"), NULL, NULL, 0, 0, 0, NULL, 0, &pi))
			{
				RETAILMSG(TRUE, (_T("ADSAutorun failed with error code %i\r\n"), GetLastError()));
				return -1;
			}
			return 0;
		case 'R':
			DEBUGMSG(TRUE, (_T("ADSAutorun Card%i Remove Detected\r\n"), sCard));

			// run local autorun app with /remove switch
			if(CreateProcess((sCard == 1) ? TEMP_FILE_NAME1 : TEMP_FILE_NAME2, _T("/REMOVE"), NULL, NULL, 0, 0, 0, NULL, 0, &pi))
			{
				// wait for it to complete
				WaitForSingleObject(pi.hProcess, INFINITE);

				// delete it
				DeleteFile((sCard == 1) ? TEMP_FILE_NAME1 : TEMP_FILE_NAME2);
			}
			return 0;
		default:
			// this shouldn't be possible, but catch it anyway to be safe
			DEBUGMSG(TRUE, (_T("ADSAutorun already registered\r\n")));
			return 0;
	}
}

short WhichCard(TCHAR *lpCmdLine)
{
	TCHAR *p;
	short card;

	p = _tcsstr(lpCmdLine, _T("DSK"));
	if(p == NULL)
	{
		return 0;
	}
	p+=3;
	p[2] = '/0';
	card = (short)_ttoi(p);

	return card;
}